@@ -1,8 +1,7 @@
 '''
 Holds implementation guts for PyHTCC
 '''
-import argparse
-import json
+import enum
 import os
 import re
 import time
@@ -23,6 +22,10 @@ class LoginCredentialsInvalidError(ValueError):
     ''' denoted if it appears as though invalid login credentials were given '''
     pass
 
+class LoginUnexpectedError(EnvironmentError):
+    ''' raised if we logged in, but the site says that there was an unexpected error'''
+    pass
+
 class TooManyAttemptsError(EnvironmentError):
     ''' raised if attempting to authenticate led to us being told we've tried too many times '''
     pass
@@ -35,6 +38,29 @@ class ZoneNotFoundError(EnvironmentError):
     ''' raised if the zone could not be found on refresh '''
     pass
 
+class SystemMode(enum.IntEnum):
+    '''
+    Enum for which mode the system is currently using
+    '''
+    EMHeat = 0
+    Heat = 1
+    Off = 2
+    Cool = 3
+    AutoHeat = 4
+    AutoCool = 5
+    SouthernAway = 6
+    Unknown = 7
+
+class FanMode(enum.IntEnum):
+    '''
+    Enum for which mode the fan is currently using
+    '''
+    Auto = 0
+    On = 1
+    Circulate = 2
+    FollowSchedule = 3
+    Unknown = 4
+
 class Zone:
     '''
     A Zone often equates to a given thermostat. The Zone object can be used to control the thermostat
@@ -80,6 +106,35 @@ class Zone:
         disp_unit = self.zone_info['DispUnits']
         return f'{raw}°{disp_unit}'
 
+    def get_system_mode(self) -> SystemMode:
+        '''
+        refreshes the cached zone information then returns the current system mode
+        '''
+        self.refresh_zone_info()
+        return SystemMode(self.zone_info['latestData']['uiData']['SystemSwitchPosition'])
+
+    def is_equipment_output_on(self) -> bool:
+        '''
+        Refreshes the cached zone information then Returns true if the EquipmentOutputStatus
+        is non 0. This typically meansthe system is heating/cooling.
+        '''
+        self.refresh_zone_info()
+        return bool(self.zone_info['latestData']['uiData']['EquipmentOutputStatus'])
+
+    def is_calling_for_heat(self) -> int:
+        '''
+        Refreshes the cached zone information and checks if the system mode is heating
+        '''
+        return self.get_system_mode() in (SystemMode.Heat, SystemMode.AutoHeat, SystemMode.EMHeat) and \
+            self.is_equipment_output_on()
+
+    def is_calling_for_cool(self) -> int:
+        '''
+        Refreshes the cached zone information and checks if the system mode is cooling
+        '''
+        return self.get_system_mode() in (SystemMode.Cool, SystemMode.AutoCool) and \
+            self.is_equipment_output_on()
+
     def get_current_temperature_raw(self) -> int:
         ''' gets the current temperature via refreshing the cached zone information '''
         self.refresh_zone_info()
@@ -93,6 +148,20 @@ class Zone:
         raw = self.get_current_temperature_raw()
         return self._get_with_unit(raw)
 
+    def get_fan_mode(self) -> FanMode:
+        '''
+        refreshes the cached zone information then returns the current FanMode
+        '''
+        self.refresh_zone_info()
+        return FanMode(self.zone_info['latestData']['fanData']['fanMode'])
+
+    def is_fan_running(self) -> bool:
+        '''
+        refreshes the cached zone information then returns True if the fan is running
+        '''
+        self.refresh_zone_info()
+        return bool(self.zone_info['latestData']['fanData']['fanIsRunning'])
+
     def get_heat_setpoint_raw(self) -> int:
         ''' refreshes the cached zone information then returns the heat setpoint '''
         self.refresh_zone_info()
@@ -215,7 +284,7 @@ class PyHTCC:
             logger.debug(f"Starting authentication attempt #{i + 1}")
             try:
                 return self._do_authenticate()
-            except (TooManyAttemptsError, RedirectDidNotHappenError) as ex:
+            except (TooManyAttemptsError, RedirectDidNotHappenError, LoginUnexpectedError) as ex:
                 logger.exception("Unable to authenticate at this moment")
                 num_seconds = 2 ** i
                 logger.debug(f"Sleeping for {num_seconds} seconds")
@@ -253,6 +322,9 @@ class PyHTCC:
         if 'portal/' not in result.url:
             raise RedirectDidNotHappenError(f"{result.url} did not represent the needed redirect")
 
+        if '/Error' in result.url:
+            raise LoginUnexpectedError(f"{result.url} denotes an error")
+
         self._set_location_id_from_result(result)
 
     def _set_location_id_from_result(self, result):
